@@ -2,11 +2,11 @@
 # coding: utf-8
 
 __author__  = "ChenyangGao <https://chenyanggao.github.io>"
-__all__ = ["get", "guess_media_type", "items", "posix_glob_translate_iter"]
+__all__ = ["get", "guess_media_type", "keys", "values", "items", "posix_glob_translate_iter"]
 
 from fnmatch import translate as wildcard_translate
 from mimetypes import guess_type
-from typing import ItemsView, Iterator, Mapping
+from typing import ItemsView, Iterator, KeysView, Mapping, ValuesView
 
 
 def get(m, /, key, default=None):
@@ -23,12 +23,30 @@ def guess_media_type(name: str, /, default: str = "application/octet-stream") ->
     return guess_type(name)[0] or default
 
 
+def keys(m, /):
+    if isinstance(m, Mapping):
+        try:
+            return m.keys()
+        except Exception:
+            return KeysView(m)
+    return m
+
+
+def values(m, /):
+    if isinstance(m, Mapping):
+        try:
+            return m.values()
+        except Exception:
+            return ValuesView(m)
+    return m
+
+
 def items(m, /):
     if isinstance(m, Mapping):
         try:
-            m = m.items()
+            return m.items()
         except Exception:
-            m = ItemsView(m)
+            return ItemsView(m)
     return m
 
 
