@@ -2,21 +2,21 @@
 # coding: utf-8
 
 __author__  = "ChenyangGao <https://chenyanggao.github.io>"
-__all__ = ["NAMESPACES", "ElementAttribProxy", "ElementProxy", "auto_property", "proxy_property"]
+__all__ = ["NAMESPACES", "PREFIXES", "ElementAttribProxy", "ElementProxy", "auto_property", "proxy_property"]
 
 from collections import UserString
 from functools import cached_property, partial
 from inspect import isclass, signature
 from re import compile as re_compile, escape as re_escape, Pattern
-from typing import Container, Final, ItemsView, Mapping, MutableMapping, Optional
+from typing import overload, Callable, Container, Final, ItemsView, Mapping, MutableMapping, Optional
 from types import MappingProxyType
 from weakref import WeakKeyDictionary, WeakValueDictionary
 
 try:
-    from lxml.etree import SubElement, _Element as Element, _ElementTree as ElementTree # type: ignore
+    from lxml.etree import register_namespace, _Element as Element, _ElementTree as ElementTree # type: ignore
     USE_BUILTIN_XML = False
 except ModuleNotFoundError:
-    from xml.etree.ElementTree import SubElement, Element, ElementTree # type: ignore
+    from xml.etree.ElementTree import register_namespace, Element, ElementTree # type: ignore
     USE_BUILTIN_XML = True
 
 from .helper import items
@@ -26,14 +26,43 @@ from .xml import el_add, el_del, el_iterfind, el_set, el_setfind, resolve_prefix
 
 
 NAMESPACES: Final = {
-    "xml": "http://www.w3.org/XML/1998/namespace", 
-    "epub": "http://www.idpf.org/2007/ops", 
-    "daisy": "http://www.daisy.org/z3986/2005/ncx/", 
-    "opf": "http://www.idpf.org/2007/opf", 
     "containerns": "urn:oasis:names:tc:opendocument:xmlns:container", 
+    "daisy": "http://www.daisy.org/z3986/2005/ncx/", 
     "dc": "http://purl.org/dc/elements/1.1/", 
+    "ds": "http://www.w3.org/2000/09/xmldsig#", 
+    "epub": "http://www.idpf.org/2007/ops", 
+    "enc": "http://www.w3.org/2001/04/xmlenc#",
+    "ncx": "http://www.daisy.org/z3986/2005/ncx/", 
+    "ns": "http://www.idpf.org/2016/encryption#compression", 
+    "opf": "http://www.idpf.org/2007/opf", 
+    "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#", 
+    "smil": "http://www.w3.org/ns/SMIL", 
+    "svg": "http://www.w3.org/2000/svg", 
+    "html": "http://www.w3.org/1999/xhtml", 
+    "wsdl": "http://schemas.xmlsoap.org/wsdl/", 
     "xhtml": "http://www.w3.org/1999/xhtml", 
+    "xml": "http://www.w3.org/XML/1998/namespace", 
+    "xs": "http://www.w3.org/2001/XMLSchema", 
+    "xsi": "http://www.w3.org/2001/XMLSchema-instance", 
 }
+# See: https://www.w3.org/TR/epub/#sec-reserved-prefixes
+PREFIXES: Final = {
+    # Package document reserved prefixes
+    "a11y": "http://www.idpf.org/epub/vocab/package/a11y/#", 
+    "dcterms": "http://purl.org/dc/terms/", 
+    "marc": "http://id.loc.gov/vocabulary/", 
+    "media": "http://www.idpf.org/epub/vocab/overlays/#", 
+    "onix": "http://www.editeur.org/ONIX/book/codelists/current.html#", 
+    "rendition": "http://www.idpf.org/vocab/rendition/#", 
+    "schema": "http://schema.org/", 
+    "xsd": "http://www.w3.org/2001/XMLSchema#", 
+    # Other reserved prefixes
+    "msv": "http://www.idpf.org/epub/vocab/structure/magazine/#", 
+    "prism": "http://www.prismstandard.org/specifications/3.0/PRISM_CV_Spec_3.0.htm#", 
+}
+
+for prefix, uri in NAMESPACES.items():
+    register_namespace(prefix, uri)
 
 
 class CachedMeta(type):
@@ -85,10 +114,11 @@ class CachedMeta(type):
         return val
 
 
-class AttributeProxy(MutableMapping):
+class AttrInfoProxy(MutableMapping):
+    __slots__ = ("__backend__",)
 
     def __init__(self, backend, /):
-        self.__backend__ = backend
+        super().__setattr__("__backend__", backend)
 
     def __contains__(self, key, /):
         return key in self.__backend__
@@ -97,6 +127,9 @@ class AttributeProxy(MutableMapping):
         del self.__backend__[key]
         return self
 
+    def __getattr__(self, attr):
+        return getattr(self.__backend__, attr)
+
     def __getitem__(self, key, /):
         return self.__backend__[key]
 
@@ -106,6 +139,9 @@ class AttributeProxy(MutableMapping):
     def __len__(self, /):
         return len(self.__backend__)
 
+    def __setattr__(self, attr, value, /):
+        raise TypeError("can't set any attributes")
+
     def __setitem__(self, key, value, /):
         self.__backend__[key] = value
         return self
@@ -153,6 +189,49 @@ class AttributeProxy(MutableMapping):
         return self
 
 
+class ElementInfoProxy(MutableMapping):
+    __slots__ = ("__backend__",)
+
+    def __init__(self, backend, /):
+        super().__setattr__("__backend__", backend)
+
+    def __repr__(self, /):
+        return "{%s}" % ", ".join("%r: %r" %t for t in zip(("tag", "attrib", "text", "tail"), self))
+
+    def __contains__(self, key, /):
+        return key in ("tag", "attrib", "text", "tail")
+
+    def __iter__(self, /):
+        backend = self.__backend__
+        return (getattr(backend, key) for key in ("tag", "attrib", "text", "tail"))
+
+    def __len__(self, /):
+        return 4
+
+    def __delitem__(self, key, /):
+        if key in ("text", "tail"):
+            setattr(self.__backend__, key, None)
+        elif key in ("tag", "attrib"):
+            raise TypeError(f"can't set key: {key!r}")
+        else:
+            raise KeyError(key)
+
+    def __getitem__(self, key, /):
+        if key in ("tag", "attrib", "text", "tail"):
+            return getattr(self.__backend__, key)
+        raise KeyError(key)
+
+    def __setitem__(self, key, value, /):
+        if key in ("text", "tail"):
+            setattr(self.__backend__, key, value)
+        else:
+            raise TypeError(f"can't set key: {key!r}")
+
+    __delattr__ = __delitem__
+    __getattr__ = __getitem__
+    __setattr__ = __setitem__
+
+
 def strip_key(key: str) -> str:
     key = key[key.rfind("}")+1:]
     key = key[key.rfind(":")+1:]
@@ -174,14 +253,14 @@ class OperationalString(UserString):
     def __iadd__(self, value, /):
         if value is not None:
             self.data += str(value)
+        return self
 
     __or__ = __add__
     __ror__ = __radd__
     __ior__ = __iadd__
 
     def __matmul__(self, value, /):
-        if value is not None:
-            self.data = str(value)
+        self.data = value
         return self
 
     def __mul__(self, n, /):
@@ -191,6 +270,7 @@ class OperationalString(UserString):
 
     def __imul__(self, n, /):
         self.data *= n
+        return self
 
     __xor__ = __radd__
     __rxor__ = __add__
@@ -199,6 +279,7 @@ class OperationalString(UserString):
         if value is None:
             return self.data
         self.data = str(value) + self.data
+        return self
 
     def __lshift__(self, value, /):
         if value is None:
@@ -208,6 +289,7 @@ class OperationalString(UserString):
     def __ilshift__(self, value, /):
         if value is not None:
             self.data = self.data.removeprefix(str(value))
+        return self
 
     def __rshift__(self, value, /):
         if value is None:
@@ -217,6 +299,7 @@ class OperationalString(UserString):
     def __irshift__(self, value, /):
         if value is not None:
             self.data = self.data.removesuffix(str(value))
+        return self
 
     def __sub__(self, value, /):
         if value is None:
@@ -226,15 +309,19 @@ class OperationalString(UserString):
     def __isub__(self, value, /):
         if value is not None:
             self.data = self.data.replace(str(value), "")
+        return self
 
     def __neg__(self, /):
         self.data = self.data.lstrip()
+        return self
 
     def __pos__(self, /):
         self.data = self.data.rstrip()
+        return self
 
     def __invert__(self, /):
         self.data = self.data.strip()
+        return self
 
     def __truediv__(self, value, /):
         if value is None:
@@ -244,6 +331,7 @@ class OperationalString(UserString):
     def __itruediv__(self, value, /):
         if value is not None and (data := self.data):
             self.data = re_compile("^(?:%s)*" % re_escape(str(value))).sub("", self.data)
+        return self
 
     def __floordiv__(self, value, /):
         if value is None:
@@ -253,6 +341,7 @@ class OperationalString(UserString):
     def __ifloordiv__(self, value, /):
         if value is not None and (data := self.data):
             self.data = re_compile("(?:%s)*$" % re_escape(str(value))).sub("", data)
+        return self
 
     def __mod__(self, value, /):
         data = self.data
@@ -267,14 +356,15 @@ class OperationalString(UserString):
             s = "(?:%s)*" % re_escape(str(value))
             data = re_compile("^"+s).sub("", data)
             self.data = re_compile(s+"$").sub("", data)
+        return self
 
 
 def auto_property(
-    key: str, 
+    key: Optional[str] = None, 
     setable: bool = False, 
     delable: bool = False, 
 ) -> property:
-    class AttribProxy(OperationalString, str):
+    class AttribProxy(OperationalString, str): # type: ignore
         __slots__ = ()
         @staticmethod
         def __init__(*args, **kwargs):
@@ -283,50 +373,63 @@ def auto_property(
         def __init_subclass__(**kwargs):
             raise TypeError("subclassing is not allowed")
         @staticmethod
-        def __getattr__(attr, /):
-            return attrib.get(key, "")
+        def __getattr__(_, /): # type: ignore
+            if key is None:
+                return instance.tail or ""
+            elif key == "":
+                return instance.text or ""
+            else:
+                return instance.get(key, "")
         @staticmethod
-        def __delattr__():
+        def __delattr__(): # type: ignore
             try:
                 deleter(instance)
             except UnboundLocalError:
                 raise TypeError("can't delete attribute")
         @staticmethod
-        def __setattr__(attr, value, /):
+        def __setattr__(_, value, /): # type: ignore
             try:
                 setter(instance, value)
             except UnboundLocalError:
                 raise TypeError("can't set attribute")
-    key = resolve_prefix(key, NAMESPACES)
+    if key:
+        key = resolve_prefix(key, NAMESPACES)
     proxy = AttribProxy()
-    instance = attrib = None
+    instance = None
     def getter(self, /):
-        nonlocal instance, attrib
+        nonlocal instance
         instance = self
-        attrib = self._attrib
         return proxy
     auto_property = property(getter)
     if setable:
         def setter(self, value, /):
-            if value is None:
-                try:
-                    deleter(self)
-                except UnboundLocalError:
-                    raise TypeError("can't delete attribute")
+            if key is None:
+                self.tail = value
+            elif key == "":
+                self.text = value
             else:
-                self._attrib[key] = str(value)
-        auto_property.setter(setter)
+                self[key] = value
+        auto_property = auto_property.setter(setter)
     if delable:
         def deleter(self, /):
-            self._attrib.pop(key, None)
-        auto_property.deleter(deleter)
+            if key is None:
+                self.tail = None
+            elif key == "":
+                self.text = None
+            else:
+                self.pop(key, None)
+        auto_property = auto_property.deleter(deleter)
     return auto_property
 
 
-def proxy_property(fget=None, /, attr: Optional[str] = "") -> property:
+@overload
+def proxy_property(fget: None, /, key: Optional[str] = "") -> Callable[[Callable], property]: ...
+@overload
+def proxy_property(fget: Callable, /, key: Optional[str] = "") -> property: ...
+def proxy_property(fget=None, /, key = ""):
     if fget is None:
-        return partial(cache_property, attr=attr)
-    class AttribProxy(OperationalString, str):
+        return partial(proxy_property, key=key)
+    class AttribProxy(OperationalString, str):  # type: ignore
         __slots__ = ()
         @staticmethod
         def __init__(*args, **kwargs):
@@ -335,48 +438,41 @@ def proxy_property(fget=None, /, attr: Optional[str] = "") -> property:
         def __init_subclass__(**kwargs):
             raise TypeError("subclassing is not allowed")
         @staticmethod
-        def __getattr__(_, /):
-            if attr is None:
-                return root.tail or ""
-            elif attr == "":
-                return root.text or ""
-            return attrib.get(attr, "")
+        def __getattr__(_, /): # type: ignore
+            if key is None:
+                return instance.tail or ""
+            elif key == "":
+                return instance.text or ""
+            else:
+                return instance.get(key, "")
         @staticmethod
-        def __delattr__():
+        def __delattr__(): # type: ignore
             deleter()
         @staticmethod
-        def __matmul__(value, /):
+        def __setattr__(_, value, /): # type: ignore
             setter(value)
-            return self
-        @staticmethod
-        def __setattr__(_, value, /):
-            setter(value)
-    if attr:
-        attr = resolve_prefix(attr, NAMESPACES)
+    if key:
+        key = resolve_prefix(key, NAMESPACES)
     proxy = AttribProxy()
-    root = attrib = None
+    instance = None
     def getter(self, /):
-        nonlocal root, attrib
-        inst = fget(self)
-        root = inst._root
-        attrib = root.attrib
+        nonlocal instance
+        instance = fget(self)
         return proxy
     def setter(value, /):
-        if value is None:
-            deleter()
-        elif attr is None:
-            root.tail = str(value)
-        elif attr == "":
-            root.text = str(value)
+        if key is None:
+            instance.tail = value
+        elif key == "":
+            instance.text = value
         else:
-            attrib[attr] = str(value)
+            instance[key] = value
     def deleter():
-        if attr is None:
-            root.tail = None
-        elif attr == "":
-            root.text = None
+        if key is None:
+            instance.tail = None
+        elif key == "":
+            instance.text = None
         else:
-            attrib.pop(attr, None)
+            instance.pop(key, None)
     return property(getter)
 
 
@@ -386,6 +482,7 @@ class ElementAttribProxy(metaclass=CachedMeta):
     __protected_keys__: tuple[str, ...] = ()
     __cache_check_key__ = lambda obj: isinstance(obj, Element)
     __cache_cls__ = WeakKeyDictionary if USE_BUILTIN_XML else WeakValueDictionary
+    __wrap_class__: "type[ElementAttribProxy]"
 
     def __init__(self, root, /):
         self._root = root
@@ -463,7 +560,6 @@ class ElementAttribProxy(metaclass=CachedMeta):
         elif isinstance(key, str):
             if not key:
                 raise ValueError("empty key not allowed")
-            key = resolve_prefix(key, self._nsmap, NAMESPACES)
             if key in self.__const_keys__ or key in self.__protected_keys__:
                 raise LookupError(f"not allowed to delete key: {key}")
             del self._attrib[key]
@@ -477,14 +573,14 @@ class ElementAttribProxy(metaclass=CachedMeta):
         return self._root is other._root
 
     def __getitem__(self, key, /):
-        if isinstance(key, (int, slice)):
-            if isinstance(key, int):
-                return type(self).wrap(self._root[key])
-            return list(map(type(self).wrap, self._root[key]))
-        elif isinstance(key, str):
+        if isinstance(key, str):
             if not key:
                 raise ValueError("empty key not allowed")
             return self._attrib[resolve_prefix(key, self._nsmap, NAMESPACES)]
+        elif isinstance(key, (int, slice)):
+            if isinstance(key, int):
+                return type(self).wrap(self._root[key])
+            return list(map(type(self).wrap, self._root[key]))
         else:
             raise TypeError("only accept `key` type: int, slice and str")
 
@@ -498,15 +594,17 @@ class ElementAttribProxy(metaclass=CachedMeta):
     def __len__(self, /):
         return len(self._attrib)
 
-    def __setitem__(self, key, val, /):
+    def __setitem__(self, key, value, /):
         if not isinstance(key, str):
-            raise TypeError("only accept `key` type: str")
+            raise TypeError("only accept `key` type: `str`")
         if not key:
             raise ValueError("empty key not allowed")
-        key = resolve_prefix(key, self._nsmap, NAMESPACES)
-        if key in self.__const_keys__:
-            raise LookupError(f"not allowed to set key: {key!r}")
-        self._attrib[key] = val
+        if value is None:
+            self.pop(key, None)
+        else:
+            if key in self.__const_keys__:
+                raise LookupError(f"not allowed to set key: {key!r}")
+            self._attrib[key] = str(value)
         return self
 
     def __repr__(self, /):
@@ -543,14 +641,29 @@ class ElementAttribProxy(metaclass=CachedMeta):
                     return wrap_class(root)
         return cls.__wrap_class__(root)
 
+    def getproxy(self, key, /):
+        if not key:
+            return
+        key = resolve_prefix(key, self._nsmap, NAMESPACES)
+        namespaces = type(self).__dict__
+        const_keys = namespaces.get("__const_keys__")
+        protected_keys = namespaces.get("__protected_keys__")
+        setable = not (const_keys and key in const_keys)
+        delable = setable and not (protected_keys and key in protected_keys)
+        return auto_property(key, setable=setable, delable=delable).fget(self)
+
     @cached_property
     def attrib(self, /):
-        return AttributeProxy(self)
+        return AttrInfoProxy(self)
 
     @property
     def nsmap(self, /):
         return self._nsmap
 
+    @cached_property
+    def info(self, /):
+        return MappingProxyType({"attrib": self.attrib})
+
     @property
     def proxy(self, /):
         return self
@@ -559,8 +672,10 @@ class ElementAttribProxy(metaclass=CachedMeta):
     def iter(self, /):
         return map(type(self).wrap, self._root.iterfind("*"))
 
-    def list(self, /):
-        return list(self.iter())
+    def list(self, /, mapfn=None):
+        if mapfn is None:
+            return list(self.iter())
+        return list(map(mapfn, self.iter()))
 
     def keys(self, /):
         return self._attrib.keys()
@@ -577,7 +692,7 @@ class ElementAttribProxy(metaclass=CachedMeta):
         attrib = self._attrib
         if const_keys or protected_keys:
             for key in tuple(attrib):
-                if key in __const_keys__ or key in protected_keys:
+                if key in const_keys or key in protected_keys:
                     continue
                 del attrib[key]
         else:
@@ -586,19 +701,21 @@ class ElementAttribProxy(metaclass=CachedMeta):
 
     def get(self, key, /, default=None):
         try:
-            return self[key]
+            return self._attrib[key]
         except LookupError:
             return default
 
     def pop(self, key, /, default=undefined):
+        if key in self.__const_keys__ or key in self.__protected_keys__:
+            raise LookupError(f"not allowed to delete key: {key}") 
         try:
-            r = self[key]
+            r = self._attrib[key]
         except LookupError:
             if default is undefined:
                 raise
             return default
         else:
-            del self[key]
+            del self._attrib[key]
             return r
 
     def popitem(self, /):
@@ -614,9 +731,9 @@ class ElementAttribProxy(metaclass=CachedMeta):
         if not isinstance(key, str):
             raise TypeError("only accept `key` type: str")
         try:
-            return self[key]
+            return seself._attriblf[key]
         except LookupError:
-            self[key] = default
+            self._attrib[key] = default
             return default
 
     def sort(self, key=id, reverse=False, use_backend_element=False):
@@ -669,6 +786,11 @@ class ElementProxy(ElementAttribProxy):
         tail = f", {tail=!r}" if tail and tail.strip() else ""
         return f"<{type(self).__qualname__}(<{self._root.tag}>{attrib}{text}{tail}) at {hex(id(self))}>"
 
+    def getproxy(self, key="", /):
+        if not key:
+            return auto_property(key, setable=True, delable=True).fget(self)
+        return super().getproxy(key)
+
     @property
     def length(self, /):
         return len(self._root)
@@ -683,7 +805,7 @@ class ElementProxy(ElementAttribProxy):
 
     @text.setter
     def text(self, text, /):
-        self._root.text = text
+        self._root.text = None if text is None else str(text)
 
     @property
     def tail(self, /):
@@ -691,7 +813,11 @@ class ElementProxy(ElementAttribProxy):
 
     @tail.setter
     def tail(self, text, /):
-        self._root.tail = text
+        self._root.tail = None if text is None else str(text)
+
+    @cached_property
+    def info(self, /):
+        return ElementInfoProxy(self)
 
     def clear(self, /):
         self._root.clear()
@@ -711,7 +837,13 @@ class ElementProxy(ElementAttribProxy):
         return type(self).wrap(el_add(self._root, name=name, attrib=attrib, text=text, tail=tail, namespaces=NAMESPACES))
 
     def delete(self, path, /):
-        el_del(self._root, path, namespaces=NAMESPACES)
+        if isinstance(path, ElementAttribProxy):
+            try:
+                self._root.remove(path._root)
+            except:
+                pass
+        else:
+            el_del(self._root, path, namespaces=NAMESPACES)
         return self
 
     def find(self, path, /):
